﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;

namespace TestProject
{
    /// <summary>
    /// 텍스트 변경시 트리거
    /// </summary>
    public class TextChangedTrigger : TriggerBase<TextBlock>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 변경 시간 속성 - ChangeTimeProperty

        /// <summary>
        /// 변경 시간 속성
        /// </summary>
        public static readonly DependencyProperty ChangeTimeProperty = DependencyProperty.Register
        (
            "ChangeTime", 
            typeof(string), 
            typeof(TextChangedTrigger), 
            new PropertyMetadata(string.Empty)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 변경 시간 - ChangeTime

        /// <summary>
        /// 변경 시간
        /// </summary>
        public string ChangeTime
        {
            get
            {
                return (string)GetValue(ChangeTimeProperty);
            }
            set
            {
                SetValue(ChangeTimeProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 부착시 처리하기 - OnAttached()

        /// <summary>
        /// 부착시 처리하기
        /// </summary>
        protected override void OnAttached()
        {
            base.OnAttached();

            AssociatedObject.Loaded += AssociatedObject_Loaded;
        }

        #endregion
        #region 탈착시 처리하기 - OnDetaching()

        /// <summary>
        /// 탈착시 처리하기
        /// </summary>
        protected override void OnDetaching()
        {
            AssociatedObject.Loaded -= AssociatedObject_Loaded;

            base.OnDetaching();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 결합 객체 로드시 처리하기 - AssociatedObject_Loaded(sender, e)

        /// <summary>
        /// 결합 객체 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void AssociatedObject_Loaded(object sender, RoutedEventArgs e)
        {
            TextBlock textBlock = AssociatedObject as TextBlock;

            if(textBlock == null)
            {
                return;
            }

            DependencyPropertyDescriptor descriptor = DependencyPropertyDescriptor.FromProperty
            (
                TextBlock.TextProperty,
                typeof(TextBlock)
            );

            if(descriptor == null)
            {
                return;
            }

            descriptor.AddValueChanged(textBlock, textBlock_TextChanged);
        }

        #endregion
        #region 텍스트 블럭 텍스트 변경시 처리하기 - textBlock_TextChanged(sender, e)

        /// <summary>
        /// 텍스트 블럭 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textBlock_TextChanged(object sender, EventArgs e)
        {
            TextBlock textBlock = sender as TextBlock;

            if(textBlock == null)
            {
                return;
            }

            if(ChangeTime.Contains(textBlock.Text))
            {
                InvokeActions("ALARM");
            }
            else
            {
                InvokeActions("");
            }
        }

        #endregion
    }
}